import sys

import click

from ..composer import DockerComposeWrapper
from ..env import SentinelHandler, _default_options
from ..util import free_ports
from . import defaultenvname


def sentinel():
    """create a redis-sentinel environment"""


@click.command()
@click.option(
    "--name",
    help="environment name",
    default=defaultenvname,
    show_default=True,
)
@click.option(
    "--force",
    help="Set, to force , if the environment exists",
    default=False,
    show_default=True,
    is_flag=True,
)
@click.option(
    "--nodes",
    "-n",
    help="Number of nodes, first node will be a standard redis node",
    default=4,
    type=int,
    show_default=True,
)
@click.option(
    "--version",
    "-v",
    help="redis version (i.e docker tag)",
    default=_default_options["_version"],
    show_default=True,
)
@click.option(
    "--image",
    "-i",
    help="Set, to specify a source docker image (without tag)",
    show_default=True,
    default=_default_options["_image"],
)
@click.option(
    "--mounts",
    "-M",
    help="directories to mount into all dockers (local remote)",
    multiple=True,
    type=(str, str),
)
@click.option("--redisconf", "-c", help="redis instance configuration file")
@click.option(
    "--user",
    "-u",
    help="Set, to specify a sentinel user",
)
@click.option(
    "--password",
    "-p",
    help="Set, to specify a sentinel password",
)
@click.option(
    "--sentinelopts",
    "-s",
    help="sentinel options, quoted. If these are specified they fully override the defaults",
    multiple=True,
    type=str,
)
@click.option(
    "--redisopts",
    "-o",
    help="redis options, quoted. These are the configuration options for the Redis (not sentinel) nodes",
    multiple=True,
    type=str,
)
@click.option(
    "--docker-ip",
    type=str,
    default="172.0.0.1",
    help="Set, to override the docker ip (i.e if you want to use an existing redis)",
)
@click.pass_context
def create(
    ctx,
    name,
    force,
    nodes,
    version,
    image,
    mounts,
    redisconf,
    user,
    password,
    sentinelopts,
    redisopts,
    docker_ip,
):
    """create and start a new environment"""

    if nodes < 4:
        sys.stderr.write(
            "Exiting. At least 4 nodes are needed for this configuration.\n"
        )
        sys.exit(3)

    ports = free_ports(nodes)
    g = SentinelHandler(ctx.obj.get("DESTDIR"), name)
    w = DockerComposeWrapper(g)

    cfg = g.gen_config(
        ports,
        user,
        password,
        sentinelopts,
        docker_ip,
    )

    sp = g.gen_spec(
        name=name,
        nodes=nodes,
        version=version,
        image=image,
        mounts=mounts,
        redisconf=redisconf,
        redisopts=redisopts,
        ports=ports,
    )

    if force:
        try:
            w.stop()
        except:
            pass
    g.start(cfg, sp)



# This layout exists, specifically to make poetry+extras work
def main():
    from .commands import cli, cluster, replica, sentinel, standalone

    cli.group(standalone.standalone).add_command(standalone.create)

    cli.group(sentinel.sentinel).add_command(sentinel.create)

    cli.group(replica.replica).add_command(replica.create)

    cli.group(cluster.cluster).add_command(cluster.create)

    cli()


if __name__ == "__main__":
    main()



import os
import shutil

import click
from loguru import logger
from ..composer import DockerComposeWrapper

from ..env import EnvironmentHandler

defaultenvname = "myenv"


@click.group()
@click.option("--debug", "-x", help="Enable debug output", default=False, is_flag=True)
@click.option("--quiet", "-q", help="Quiet all outputs", default=False, is_flag=True)
@click.option(
    "--dest",
    "-d",
    help="Directory for environment files",
    default=os.path.join(os.getcwd(), "redisenv.envs"),
    show_default=True,
)
@click.option(
    "--templates",
    "-t",
    help="Directory for environment templates",
    default=os.path.join(os.getcwd(), "redisenv.templates"),
    show_default=True,
)
@click.pass_context
def cli(ctx, debug, quiet, dest, templates):
    ctx.ensure_object(dict)
    ctx.obj["DEBUG"] = debug
    ctx.obj["DESTDIR"] = dest
    ctx.obj["TEMPLATEDIR"] = templates
    if debug:
        logger.level("DEBUG")
    if quiet:
        logger.level("CRITICAL")


@cli.command()
@click.option(
    "--name",
    help="environment name",
    default=defaultenvname,
    show_default=True,
)
@click.pass_context
def destroy(ctx, name):
    """destroy an environment"""
    g = EnvironmentHandler(ctx.obj.get("DESTDIR"), name)
    w = DockerComposeWrapper(g)
    w.stop()
    os.unlink(g.envfile)
    tree = os.path.join(g.envdir, name)
    if os.path.isdir(tree):
        shutil.rmtree(tree)


@cli.command()
@click.option(
    "--name",
    help="environment name",
    default=defaultenvname,
    show_default=True,
)
@click.pass_context
def pause(ctx, name):
    """pause an environment"""
    g = EnvironmentHandler(ctx.obj.get("DESTDIR"), name)
    w = DockerComposeWrapper(g)
    w.pause()


@cli.command()
@click.option(
    "--name",
    help="environment name",
    default=defaultenvname,
    show_default=True,
)
@click.pass_context
def unpause(ctx, name):
    """unpause an environment"""
    g = EnvironmentHandler(ctx.obj.get("DESTDIR"), name)
    w = DockerComposeWrapper(g)
    w.unpause()


@cli.command()
@click.option(
    "--name",
    help="environment name",
    default=defaultenvname,
    show_default=True,
)
@click.pass_context
def restart(ctx, name):
    """restart an environment"""
    g = EnvironmentHandler(ctx.obj.get("DESTDIR"), name)
    w = DockerComposeWrapper(g)
    w.restart()


@cli.command()
@click.pass_context
def listenvs(ctx):
    """list environments"""
    g = EnvironmentHandler(ctx.obj.get("DESTDIR"), "")
    g.listenvs()


@cli.command()
@click.option(
    "--name",
    help="environment name",
    default=defaultenvname,
    show_default=True,
)
@click.pass_context
def ports(ctx, name):
    """list the ports generated for an environment"""
    g = EnvironmentHandler(ctx.obj.get("DESTDIR"), name)
    g.listports()



import socket
from typing import List

from loguru import logger


def free_ports(num: int, cluster: bool = False) -> List:
    """Find free ports, for the number of nodes specified
    starting at the base port.
    """

    ports = []
    while len(ports) != num:
        s = socket.socket()
        s.bind(("", 0))
        port = s.getsockname()[1]
        s.close()

        # redis limitation
        if port >= 55535:
            continue

        s = socket.socket()
        try:
            s.bind(("", port + 10000))
        except OSError:  # port alread in use
            continue
        s.close()
        ports.append(port)
    return ports



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['redisenv', 'redisenv.commands']

package_data = \
{'': ['*'], 'redisenv': ['templates/*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'jinja2>=3.1.2,<4.0.0',
 'loguru>=0.6.0,<0.7.0',
 'pyyaml>=6.0,<7.0']

entry_points = \
{'console_scripts': ['redisenv = redisenv:__main__.main']}

setup_kwargs = {
    'name': 'redisenv',
    'version': '0.1.3',
    'description': 'A tool for building redis test environments',
    'long_description': '# redisenv\n\n[![MIT licensed](https://img.shields.io/badge/license-MIT-blue.svg)](./LICENSE)\n[![pypi](https://badge.fury.io/py/redisenv.svg)](https://pypi.org/project/redisenv/)\n[![CI](https://github.com/RedisLabsModules/redisenv/actions/workflows/integration.yml/badge.svg)](https://github.com/RedisLabsModules/redisenv/actions/workflows/integration.yml)\n\nredisenv is library that makes generating redis test environments easier. This tool generates [docker-compose](https://docs.docker.com/compose/) files, and runs all instances within docker.  Generated docker instances select a random port, based on the available free ports, and when running ```redisenv ports```, a JSON parse-able string of the build connections is outputted.\n\nCurrently redisenv supports:\n\n* Redis standalone\n\n* Redis Sentinel\n\n* Redis Clusters\n\n* Redis masters with replicas\n\nNote: Today Redis Standalone supports redis-stack, but nothing else does\n\n----\n\n## Installation\n\n### Requirements\n\n* Python >= 3.7\n\n* docker-compose\n\n* docker\n\n## Usage\n\nList options. Note, each sub command accepts its own ```--help```\n\n```bash\nredisenv --help\n```\n\nStart an environment named foo, with one container:\n\n```bash\nredisenv --name foo standalone create --nodes 1\n```\n\nStart an environment with the redisbloom module, downloaded into a folder named modules. Note - you need the full *local* path to the directory.\n\n```bash\nredisenv --name foo standalone create -n 1 -M `pwd`/modules /modules -o "--loadmodule /modules/redisbloom.so"\n```\n\nDestroy the environment named foo:\n\n```bash\nredisenv --name foo destroy\n```\n',
    'author': 'Redis OSS',
    'author_email': 'oss@redis.io',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import sys

import click

from ..env import ClusterHandler, _default_options
from ..util import free_ports
from ..composer import DockerComposeWrapper
from . import defaultenvname


def cluster():
    """for creating an OSS redis cluster environment"""


@click.command()
@click.option(
    "--name",
    help="environment name",
    default=defaultenvname,
    show_default=True,
)
@click.option(
    "--force",
    help="Set, to force , if the environment exists",
    default=False,
    show_default=True,
    is_flag=True,
)
@click.option(
    "--nodes",
    "-n",
    help="Number of nodes, first node will be a standard redis node",
    default=3,
    type=int,
    show_default=True,
)
@click.option(
    "--version",
    "-v",
    help="redis version (i.e docker tag)",
    default=_default_options["_version"],
    show_default=True,
)
@click.option(
    "--image",
    "-i",
    help="Set, to specify a source docker image (without tag)",
    show_default=True,
    default=_default_options["_image"],
)
@click.option(
    "--mounts",
    "-M",
    help="directories to mount into all dockers (local remote)",
    multiple=True,
    type=(str, str),
)
@click.option(
    "--redisopts",
    "-o",
    help="redis options, quoted - passed to all containers",
    multiple=True,
    type=str,
)
@click.option(
    "--replicas",
    help="number of replicas in the cluster",
    default=_default_options["_cluster_replicas"],
    type=int,
)
@click.pass_context
def create(
    ctx,
    name,
    force,
    nodes,
    version,
    image,
    mounts,
    redisopts,
    replicas,
):
    """create and start a new environment"""

    if nodes < 3:
        sys.stderr.write(
            "Exiting. At least 3 nodes are needed for this configuration.\n"
        )
        sys.exit(3)

    if replicas >= nodes:
        sys.stderr.write("There must be fewer replicas than nodes.\n")
        sys.exit(3)

    ports = free_ports(nodes, cluster=True)
    g = ClusterHandler(ctx.obj.get("DESTDIR"), name)
    w = DockerComposeWrapper(g)

    cfg = g.gen_config(ports, redisopts)

    sp = g.gen_spec(
        nodes,
        version,
        image,
        mounts,
        ports,
        replicas,
    )

    if force:
        try:
            w.stop()
        except:
            pass
    g.start(cfg, sp)
